@@ -47,26 +47,54 @@ from chainlib.eth.settings import process_settings
 logg = logging.getLogger()
 
 
+def from_data_arg(data):
+    try:
+        data = strip_0x(data)
+        logg.info('appended input hex data: {}'.format(data))
+        return data
+    except ValueError:
+        pass
+
+    filename = data
+    try:
+        f = open(filename, 'r')
+    except:
+        raise ValueError("data not hex and not file we can open")
+
+    data = f.read()
+    f.close()
+    logg.info('appended input data from file {}: {}'.format(filename, data))
+
+    data = data.rstrip()
+    return strip_0x(data)
+
+
 def process_config_local(config, arg, args, flags):
-    config.add(args.data, '_DATA', False)
-    config.add(args.amount, '_VALUE', False)
+    data = ''
+    if args.data != None:
+        for data_arg in args.data:
+            data += from_data_arg(data_arg)
+    if data == '':
+        data = None
+    config.add(data, '_DATA', False)
+    config.add(args.amount[0], '_VALUE', False)
     return config
 
 
 arg_flags = ArgFlag()
 arg = Arg(arg_flags)
-flags = arg_flags.STD_WRITE | arg_flags.WALLET
+flags = arg_flags.STD_WRITE | arg_flags.WALLET | arg_flags.CREATE | arg_flags.VALUE
 
 argparser = chainlib.eth.cli.ArgumentParser()
 argparser = process_args(argparser, arg, flags)
-argparser.add_argument('--data', type=str, help='Transaction data')
+argparser.add_argument('--data', type=str, action='append', help='Transaction data')
 argparser.add_argument('amount', type=str, help='Token amount to send')
 args = argparser.parse_args()
 
 logg = process_log(args, logg)
 
 config = Config()
-config = process_config(config, arg, args, flags)
+config = process_config(config, arg, args, flags, positional_name='amount')
 config = process_config_local(config, arg, args, flags)
 logg.debug('config loaded:\n{}'.format(config))
 
@@ -90,13 +118,11 @@ def main():
     g = Gas(
             settings.get('CHAIN_SPEC'),
             signer=settings.get('SIGNER'),
-            gas_oracle=settings.get('GAS_ORACLE'),
+            gas_oracle=settings.get('FEE_ORACLE'),
             nonce_oracle=settings.get('NONCE_ORACLE'),
             )
 
-    recipient = to_checksum_address(config.get('_RECIPIENT'))
-    if not config.true('_UNSAFE') and not is_checksum_address(recipient):
-        raise ValueError('invalid checksum address')
+    recipient = settings.get('RECIPIENT')
 
     if logg.isEnabledFor(logging.DEBUG):
         try:
@@ -105,13 +131,14 @@ def main():
                     settings.get('SENDER_ADDRESS'),
                     settings.get('RPC_ID_GENERATOR'),
                     )
-            recipient_balance = balance(
-                    settings.get('CONN'),
-                    settings.get('RECIPIENT'),
-                    settings.get('RPC_ID_GENERATOR'),
-                    )
             logg.debug('sender {} balance before: {}'.format(settings.get('SENDER_ADDRESS'), sender_balance))
-            logg.debug('recipient {} balance before: {}'.format(settings.get('RECIPIENT'), recipient_balance))
+            if settings.get('RECIPIENT') != None:
+                recipient_balance = balance(
+                        settings.get('CONN'),
+                        settings.get('RECIPIENT'),
+                        settings.get('RPC_ID_GENERATOR'),
+                        )
+                logg.debug('recipient {} balance before: {}'.format(settings.get('RECIPIENT'), recipient_balance))
         except urllib.error.URLError:
             pass
      
@@ -119,7 +146,7 @@ def main():
             settings.get('SENDER_ADDRESS'),
             settings.get('RECIPIENT'),
             settings.get('VALUE'),
-            data=config.get('_DATA'),
+            data=settings.get('DATA'),
             id_generator=settings.get('RPC_ID_GENERATOR'),
         )
     
@@ -135,13 +162,14 @@ def main():
                         settings.get('SENDER_ADDRESS'),
                         settings.get('RPC_ID_GENERATOR'),
                         )
-                recipient_balance = balance(
-                        settings.get('CONN'),
-                        settings.get('RECIPIENT'),
-                        settings.get('RPC_ID_GENERATOR'),
-                        )
                 logg.debug('sender {} balance before: {}'.format(settings.get('SENDER_ADDRESS'), sender_balance))
-                logg.debug('recipient {} balance before: {}'.format(settings.get('RECIPIENT'), recipient_balance))
+                if settings.get('RECIPIENT') != None:
+                    recipient_balance = balance(
+                            settings.get('CONN'),
+                            settings.get('RECIPIENT'),
+                            settings.get('RPC_ID_GENERATOR'),
+                            )
+                    logg.debug('recipient {} balance before: {}'.format(settings.get('RECIPIENT'), recipient_balance))
             if r['status'] == 0:
                 logg.critical('VM revert for {}. Wish I could tell you more'.format(tx_hash_hex))
                 sys.exit(1)
